/*
 * Account.java
 *
 * Created on February 7, 2002, 11:55 AM
 */

package com.systinet.demos.bank.simple;

import com.systinet.demos.bank.Account;
import com.systinet.demos.bank.Balance;
import com.systinet.demos.bank.Transfer;

import com.systinet.demos.bank.BalanceRetrievalException;
import com.systinet.demos.bank.TransferProcessingException;
import com.systinet.demos.bank.TransferRetrievalException;


/**
 *
 * @author  zdenek
 */
public class AccountImpl implements Account {
    
     /** Holds value of property accountNumber. */
    public String accountNumber;    

    /** Holds value of property ssn. */
    public String ssn;
    
    /** Holds value of property type. */
    public String type;
    
    /** Holds value of property balanceHistory. */
    public Balance[] balanceHistory = null;
    
    /** Holds value of property transferHistory. */
    public Transfer[] transferHistory = null;
    
    /** Holds value of property lastBalance. */
    public Balance lastBalance = null;
    
    /** Holds value of property lastTransfer. */
    public Transfer lastTransfer = null;
    
    /** Creates a new instance of Account */
    public AccountImpl() {
    }
    
    /** Getter for property accountNumber.
     * @return Value of property accountNumber.
     */
    public String getAccountNumber() {
        return this.accountNumber;
    }
    
    /** Setter for property accountNumber.
     * @param accountNumber New value of property accountNumber.
     */
    public void setAccountNumber(String accountNumber) {
        this.accountNumber = accountNumber;
    }
    
    /** Getter for property SSN.
     * @return Value of property SSN.
     */
    public String getSSN() {
        return this.ssn;
    }
    
    /** Setter for property SSN.
     * @param ssn New value of property SSN.
     */
    public void setSSN(String ssn) {
        this.ssn = ssn;
    }
    
    /** Getter for property type.
     * @return Value of property type.
     */
    public String getType() {
        return this.type;
    }
    
    /** Setter for property type.
     * @param type New value of property type.
     */
    public void setType(String type) {
        this.type = type;
    }
    
    /** Getter for property lastBalance.
     * @return Value of property lastBalance.
     */
    public Balance getLastBalance() {
        return this.lastBalance;
    }  

    /** Setter for property lastBalance.
     * @param type New value of property lastBalance.
     */
    public void setLastBalance(Balance balance) {
        this.lastBalance = balance;
    }
    
    /** Getter for property lastTransfer.
     * @return Value of property lastTransfer.
     */
    public Transfer getLastTransfer() {
        return this.lastTransfer;
    }  

    /** Setter for property lastTransfer.
     * @param type New value of property lastTransfer.
     */
    public void setLastTransfer(Transfer transfer) {
        this.lastTransfer = transfer;
    }
    
    public Balance processTransfer(Transfer transfer) throws TransferProcessingException {
        try {
            Balance balance = (Balance)this.getLastBalance();
            double current = 0;
            if(balance != null) {
                current = balance.getBalance();
            }
            double amount = transfer.getAmount();
            String acc = transfer.getAccountNumber();
            String accBen = transfer.getAccountNumberBeneficiary();
            if(((amount > 0) && ((accBen == null)||(!accBen.equalsIgnoreCase(this.accountNumber)))) ||
                ((amount < 0) && ((acc == null)||(!acc.equalsIgnoreCase(this.accountNumber))))) {
                throw new TransferProcessingException("Can't process transfer. Reason: Invalid account number.");
            }
            if((amount>=0) || (current + amount >= 0)) {
                java.sql.Date dt = new java.sql.Date((new java.util.Date()).getTime());
                long id = SimpleIDGenerator.createID();
                balance = new BalanceImpl();
                balance.setAccountNumber(this.accountNumber);
                balance.setChanged(dt);
                transfer.setTimestamp(new java.sql.Date(dt.getTime()));
                balance.setBalance(current + amount);
                balance.setComment(transfer.getComment());
                balance.setTransferID(id);
                transfer.setTransferID(id);
                balance.setTransfer(transfer);
                transfer.setBalance(balance);
                this.lastTransfer = transfer;
                this.lastBalance = balance;
                int cnt = 0;
                if(this.balanceHistory != null)
                    cnt = balanceHistory.length;
                Balance[] newBalanceHistory = new Balance[cnt+1];
                if(this.balanceHistory != null) {
                    System.arraycopy(this.balanceHistory, 0, newBalanceHistory, 0, cnt);
                }
                newBalanceHistory[cnt] = balance;
                this.balanceHistory = newBalanceHistory;
                
                cnt = 0;
                if(this.transferHistory != null)
                    cnt = transferHistory.length;
                Transfer[] newTransferHistory = new Transfer[cnt+1];
                if(this.transferHistory != null) {
                    System.arraycopy(this.transferHistory, 0, newTransferHistory, 0, cnt);
                }
                newTransferHistory[cnt] = transfer;
                this.transferHistory = newTransferHistory;
                
                return balance;
            }
            else {
                throw new TransferProcessingException("Can't process transfer. Reason: Unsufficient funds.");
            }
        }
        catch (Exception e) {
            e.printStackTrace();
            throw new TransferProcessingException("Can't process transfer. Reason: "+e.getMessage());
        }
    }
    
    
    /** Getter for property balanceHistory.
     * @return Value of property balanceHistory.
     */
    public Balance[] getBalanceHistory() {
        return this.balanceHistory;
    }
    
    /** Setter for property balanceHistory.
     * @param type New value of property balanceHistory.
     */
    public void setBalanceHistory(Balance[] balanceHistory) {
        this.balanceHistory = balanceHistory;
    }
        
    /** Getter for property transferHistory.
     * @return Value of property transferHistory.
     */
    public Transfer[] getTransferHistory() {
        return this.transferHistory;
    }
    
    /** Setter for property transferHistory.
     * @param type New value of property transferHistory.
     */
    public void setTransferHistory(Transfer[] transferHistory) {
        this.transferHistory = transferHistory;
    }    
    
    
}