package com.systinet.demos.ws;

import com.systinet.demos.bank.Customer;
import com.systinet.demos.bank.Account;
import com.systinet.demos.bank.Balance;
import com.systinet.demos.bank.Transfer;

import com.systinet.demos.bank.TransferProcessingException;
import com.systinet.demos.bank.CustomerNotFoundException;
import com.systinet.demos.bank.BalanceRetrievalException;
import com.systinet.demos.bank.AccountRetrievalException;

import com.systinet.demos.bank.simple.CustomerFactoryImpl;

public class Bank {
    
    
    private static CustomerFactoryImpl factory = new CustomerFactoryImpl();
    
    public synchronized String[] getAccountNumbers(String ssn) throws CustomerNotFoundException, AccountRetrievalException {
        Customer customer = factory.createCustomer(ssn);
        Account[] accounts = customer.getAccounts();
        String[] numbers = new String[accounts.length];
        for(int i=0; i<accounts.length; i++) {
            numbers[i] = accounts[i].getAccountNumber();
        }
        return numbers;
    }
    
    public synchronized double getTotalAmount(String ssn) throws CustomerNotFoundException, AccountRetrievalException, BalanceRetrievalException {
        Customer customer = factory.createCustomer(ssn);
        Account[] accounts = customer.getAccounts();
        double sum = 0;
        for(int i=0; i<accounts.length; i++) {
            Balance balance = accounts[i].getLastBalance();
            if(balance != null) {
                sum += balance.getBalance();
            }
        }
        return sum;
    }
    
    public synchronized void submitTransaction(String ssn, Transfer transfer) throws CustomerNotFoundException, AccountRetrievalException, TransferProcessingException {
        double amount = transfer.getAmount();
        String accountNumber = (amount >= 0)?(transfer.getAccountNumberBeneficiary()):(transfer.getAccountNumber());
        Customer customer = factory.createCustomer(ssn);
        Account[] accounts = customer.getAccounts();
        Account account = null;
        for(int i=0; i<accounts.length; i++) {
            if(accountNumber.equalsIgnoreCase(accounts[i].getAccountNumber())) {
                account = accounts[i];
                break;
            }
        }
        if(account != null) {
            account.processTransfer(transfer);
        }
        else {
            throw new AccountRetrievalException("Account with account number "+accountNumber+" doesn't exist.");
        }
    }
 
    public synchronized double getAccountBalance(String ssn, String accountNumber) throws CustomerNotFoundException, AccountRetrievalException, BalanceRetrievalException {
        Customer customer = factory.createCustomer(ssn);
        Account[] accounts = customer.getAccounts();
        Account account = null;
        for(int i=0; i<accounts.length; i++) {
            if(accountNumber.equalsIgnoreCase(accounts[i].getAccountNumber())) {
                account = accounts[i];
                break;
            }
        }
        
        if(account != null) {
            Balance balance = account.getLastBalance();
            if(balance != null) {
                return balance.getBalance();
            }
            else {
                return 0;
            }
        }
        else {
            throw new AccountRetrievalException("Account with account number "+accountNumber+" doesn't exist.");
        }
            
    }
}